"use strict";
/**
 * async-cache-queue.ts
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearCache = exports.queueFactory = void 0;
const cache_1 = require("./cache");
const concurrency_1 = require("./concurrency");
const timeout_1 = require("./timeout");
const container_1 = require("./container");
/**
 * It returns a function to create a queue manager.
 */
function queueFactory(options) {
    const { cache, concurrency, negativeCache, storage, timeout, timeoutFallback } = options || {};
    let flow = null;
    if (timeout) {
        flow = join(flow, (0, timeout_1.timeoutFactory)(timeout, timeoutFallback));
    }
    if (concurrency) {
        flow = join(flow, (0, concurrency_1.concurrencyFactory)(concurrency));
    }
    if (cache || negativeCache || storage) {
        flow = join(flow, (0, cache_1.cacheFactory)(options));
    }
    if (!flow) {
        flow = fn => arg => Promise.resolve().then(() => fn(arg));
    }
    return fn => flow(fn);
}
exports.queueFactory = queueFactory;
function join(prev, filter) {
    return (prev && filter) ? (fn => filter(prev(fn))) : (prev || filter);
}
/**
 * It purges all cache storage handled in the library.
 */
function clearCache() {
    (0, container_1.clearContainers)();
}
exports.clearCache = clearCache;
