"use strict";
/**
 * concurrency.ts
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.concurrencyFactory = void 0;
function concurrencyFactory(concurrency) {
    if (!(concurrency > 0)) {
        throw new Error("Invalid concurrency: " + concurrency);
    }
    return fn => {
        const queue = [];
        let running = 0;
        return arg => new Promise((resolve, reject) => {
            const job = () => {
                running++;
                return Promise.resolve().then(() => fn(arg)).then(resolve, reject);
            };
            queue.push(job);
            next();
        });
        function next() {
            if (running >= concurrency)
                return;
            const job = queue.shift();
            if (job)
                job().then(done, done);
        }
        function done() {
            running--;
            next();
        }
    };
}
exports.concurrencyFactory = concurrencyFactory;
