"use strict";
/**
 * timeout.ts
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.timeoutFactory = void 0;
function timeoutFactory(timeout, fallback) {
    if (!(timeout > 0)) {
        throw new Error("Invalid timeout: " + timeout);
    }
    if (fallback == null) {
        fallback = () => Promise.reject(new Error("timeout: " + timeout + "ms"));
    }
    if ("function" !== typeof fallback) {
        throw new Error("Invalid timeoutFallback: " + fallback);
    }
    return fn => arg => new Promise((resolve, reject) => {
        let count = 0;
        let timer = setTimeout(() => {
            timer = null;
            if (!count++)
                return Promise.resolve().then(() => fallback(arg)).then(resolve, reject);
        }, timeout);
        Promise.resolve().then(() => fn(arg)).then(result => {
            if (timer)
                clearTimeout(timer);
            if (!count++)
                resolve(result);
        }, reason => {
            if (timer)
                clearTimeout(timer);
            if (!count++)
                reject(reason);
        });
    });
}
exports.timeoutFactory = timeoutFactory;
